/* 1-wire connection demo to a HD44780-compatible LCD via shiftregister

ShiftRegLCD123 wiring
  Requires 1, 2 or 3 pins from the Arduino.
  Also arduinoshiftreglcd / ShiftRegLCD compatible, except added 1-wire mode.

SR output:  SRLCD123 wiring:
  Bit  #0   - N/C - not connected. Must be 0 in 1-wire and 2-wire mode.
  Bit  #1   - N/C or LCD backlight. Do not connect directly!
              Use a driver / transistor!
  Bit  #2   - connects to RS (Register Select) on the LCD
  Bits #3-6 - connects to LCD data inputs D4 - D7.
  Bit  #7   - 1-wire mode: To LCD Enable input.
              2-wire mode: To diode cathode of the diode-resistor AND "gate"
              3-wire mode: N/C, unused. LCD Enable directly from Arduino.
  LCD R/!W-pin hardwired to LOW (only writing to LCD).

USAGE:

  1: Make an LCD object, set arduino output pins and LCD wiring scheme:

        ShiftRegLCD123 LCDobject( Datapin [, scheme] )
    or  ShiftRegLCD123 LCDobject( Datapin , Clockpin, scheme )
    or  ShiftRegLCD123 LCDobject( Datapin , Clockpin, Latchpin, scheme )

    where:
      Datapin : Arduino pin to shiftregister serial data input.
      Clockpin: Arduino pin to shiftregister clock input.
      Latchpin: Arduino pin to shiftregister latch/strobe/register clock input.
      scheme  : SRLCD123 or LCD3WIRE.
                Indicates shiftregister to LCD wiring type / variant.

  2: Initialize the LCD by calling begin() function with LCD size and font:

        LCDobject.begin( cols, lines [, font] )

    where:
      cols    : Nr. of columns in the LCD
      lines   : Nr. of "logical lines" in the LCD (not neccesarily physical)
      font    : 0 = small (default), 1 = large font for some 1-line LCD's only.
*/

#include <ShiftRegLCD123.h>
#include <SoftwareSerial.h>

// LCD signals from Arduino pin 1
const byte lcdPin  = 1;

// Instantiate a LCD object, using SRLCD123 wiring as default
ShiftRegLCD123 srlcd(lcdPin);
SoftwareSerial mySerial(2,0);
//====================================
byte armsDown[8] = {
  0b00100,
  0b01010,
  0b00100,
  0b01110,
  0b10101,
  0b00100,
  0b01010
};

byte armsUp[8] = {
  0b00100,
  0b01010,
  0b10101,
  0b01110,
  0b00100,
  0b00100,
  0b01010
};

//====================================
void setup() {

pinMode(4,INPUT); // Пин PB4. Номер 3 физический.   
  
  // Initialize LCD and set display size
  // LCD size 20 columns x 2 lines, small (normal) font
  srlcd.begin(16,2);
  delay(100);
  mySerial.begin(9600);
  delay(100); 
  
  //Print a message to the LCD.
  srlcd.print("! HELLO, WORLD !");

  // move to next line
  srlcd.setCursor(0,1);
  srlcd.print("Ariadna-on-Line!");

    // create a new character
  srlcd.createChar(0, armsDown);
  // create a new character
  srlcd.createChar(1, armsUp);

  delay(3000);
}
//====================================
void loop() {

  // read the potentiometer on A0:
int sensorReading = analogRead(2);  //Пин PB4. Он же ADC2. 
  // map the result to 200 - 1000:
int delayTime = map(sensorReading, 0, 1023, 100, 1000);  

  // Print a message to the LCD.
//    srlcd.setCursor(0,0);
//  srlcd.print("! HELLO, WORLD !");
  // move to next line
//  srlcd.setCursor(0,1);
//  srlcd.print("LCD 1- wire test");
  
 // Turn on backlight (if used)
  srlcd.backlightOn();
  mySerial.println("BackLight ON");
  srlcd.setCursor(0,0);
  srlcd.write(0);  
  srlcd.buzzerOn();
  delay(delayTime);
  srlcd.setCursor(0,0);
  srlcd.write(1);  
//  srlcd.clear();
  srlcd.backlightOff();
  mySerial.println("BackLight OFF");
  
  srlcd.buzzerOff();  
  delay(delayTime);  
}
//====================================
